<?php
/**
 * 2007-2017 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2017 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * EstoAPI Class
 * Responsible for communication with ESTO API
 */
class EstoAPI
{

    const API_ENDPOINTS = [
        CountryCodeType::ESTONIA => 'https://api.esto.ee/v2/',
        CountryCodeType::LITHUANIA => 'https://api.estopay.lt/v2/',
        CountryCodeType::LATVIA => 'https://api.esto.lv/v2/',
    ];

    private $apiURL;
    private $esto_shop_id = '';
    private $esto_secret_key = '';
    private $api_country = '';

    public function __construct($apiURL, $esto_shop_id=null, $esto_secret_key=null, $api_country=null)
    {
        $this->apiURL = $apiURL;

        if (!empty($esto_shop_id) && !empty($esto_secret_key) && !empty($api_country)) {
            $this->esto_shop_id = $esto_shop_id;
            $this->esto_secret_key = $esto_secret_key;
            $this->api_country = $api_country;
        } else {
            $this->setDefaultCountryApiDetails();
        }
    }

    public function getPurchaseRedirect($data)
    {
        $service = 'purchase';
        $method = 'POST';

        $response = $this->makeCall($service, $data, $method);

        return $response->purchase_url;
    }

    public function getMonthlyPayment($data)
    {
        $service = 'calculate/payment';
        $method = 'POST';

        $response = $this->makeCall($service, $data, $method);

        return $response->monthly_payment;
    }

    /**
     * Method for handling calls to ESTO API
     */
    private function makeCall($service, $data = [], $method = 'GET')
    {
        $url = $this->apiURL . $service;

        if ($method === 'GET') {
            $url .= '?' . http_build_query($data);
        }

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        // Basic auth
        $user = $this->esto_shop_id;
        $password = $this->esto_secret_key;
        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($ch, CURLOPT_USERPWD, $user . ':' . $password);

        $headers = array(
            'Accept: application/json',
            'Content-Type: application/json',
        );
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $data = json_encode($data);

        switch ($method) {
            case 'GET':
                break;

            case 'POST':
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
                break;

            case 'PUT':
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
                curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
                break;

            case 'DELETE':
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
                break;
        }

        $response = curl_exec($ch);
        $response = json_decode($response);

        if (isset($response->data)) {
            return json_decode($response->data);
        } else {
            return $response;
        }
    }

    /**
     * Method, that returns payment methods that are turned on from side of ESTO backend
     */
    public function getAvailablePaymentMethods()
    {
        $connectionMode = Configuration::get(ConfigKeysHelper::ESTO_CONNECTION_MODE);

        $isConnectionTestMode = (int) ($connectionMode === ConnectionModeType::TEST);

        $primaryCountry = Configuration::get(ConfigKeysHelper::ESTO_COUNTRY);

        $countryCode = CountryHelper::API_COUNTRIES[$primaryCountry];

        $curl = curl_init();

        curl_setopt($curl, CURLOPT_URL, $this->apiURL . 'purchase/payment-methods?country_code=' . $countryCode . '&test_mode=' . $isConnectionTestMode);

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        // Basic auth
        $user = $this->esto_shop_id;
        $password = $this->esto_secret_key;

        curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($curl, CURLOPT_USERPWD, $user . ':' . $password);

        $headers = array(
            'Accept: application/json',
            'Content-Type: application/json'
        );
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);


        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        $auth = curl_exec($curl);
        curl_close($curl);

        $result = json_decode($auth,true);

        $methods[] = '';
        foreach ($result as $value) {
            if ($value['schedule_type'] === PaymentScheduleTypeType::REGULAR) {
                $methods[] = PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::REGULAR];
            } elseif ($value['schedule_type'] === PaymentScheduleTypeType::ESTO_X) {
                $methods[] = PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X];
            } elseif ($value['schedule_type'] === PaymentScheduleTypeType::PAY_LATER) {
                $methods[] = PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::PAY_LATER];
            } elseif ($value['type'] === PaymentTypeType::BANKLINK) {
                $methods[] = PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK];
            } elseif ($value['type'] === PaymentTypeType::CARD) {
                $methods[] = PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD];
            }
        }

        return $methods;
    }

    /**
     * Method, that returns all banklinks available for merchant
     */
    public function getBanklinks($data = null) {

        $service = isset($data['service']) ? $data['service'] : 'purchase';

        $url = $this->apiURL . $service;

        $connectionMode = Configuration::get(ConfigKeysHelper::ESTO_CONNECTION_MODE);

        $isConnectionTestMode = (int) ($connectionMode === ConnectionModeType::TEST);

        $banks = [];
        foreach (CountryHelper::API_COUNTRIES as $v) {

            $countryCode = $v;

            $curl = curl_init();

            curl_setopt($curl, CURLOPT_URL, $url . '/payment-methods?country_code=' . $countryCode . '&test_mode=' . $isConnectionTestMode);

            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

            // Basic auth
            $user = $this->esto_shop_id;
            $password = $this->esto_secret_key;

            curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
            curl_setopt($curl, CURLOPT_USERPWD, $user . ':' . $password);

            $headers = array(
                'Accept: application/json',
                'Content-Type: application/json'
            );
            curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);


            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            $auth = curl_exec($curl);
            curl_close($curl);

            $return = json_decode($auth,true);

            $returnArray = [];
            foreach ($return as $k => $value) {

                // select only banklinks
                if ($value['type'] !== PaymentTypeType::BANKLINK) {
                    continue;
                }

                // delete items which have is_test = true for live mode
                if (!$isConnectionTestMode && $value['is_test']) {
                    continue;
                }

                $returnArray[$k] = $value;
            }

            $banks[$countryCode] = $returnArray;

        }

        return $banks;

    }

    protected function setDefaultCountryApiDetails()
    {
        $api_details = Esto::getDefaultCountryApiData();
        if (empty($api_details)) {
            return;
        }

        $this->apiURL = $api_details['url'];
        $this->esto_shop_id = $api_details['user'];
        $this->esto_secret_key = $api_details['pass'];
        $this->api_country = $api_details['country_iso'];
    }
}
