<?php

/**
 * EstoInstall Class
 * Responsible for install/uninstall of module
 */
class EstoInstall
{

    public function install()
    {
        $this->createOrderStates();
        $this->addUniqueConstraintToCartIdInOrdersTable();

        if ($this->isSomeValuesNotSetup()) {
            return false;
        }

        return $this->installDefaultLanguageConfiguration();
    }

    public function uninstall()
    {
        $configs = ConfigKeysHelper::getConstants();

        foreach ($configs as $c) {
            Configuration::deleteByName($c);
        }

        $this->uninstallBanksTable();
        $this->removeUniqueConstraintToCartIdInOrdersTable();

        return true;
    }

    private function isSomeValuesNotSetup()
    {
        return !Configuration::updateValue(ConfigKeysHelper::ESTO_SHOP_ID_EE, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_SECRET_KEY_EE, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_SHOP_ID_LV, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_SECRET_KEY_LV, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_SHOP_ID_LT, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_SECRET_KEY_LT, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_CONNECTION_MODE, ConnectionModeType::LIVE)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_CREATE_ORDER, CreateOrderType::AFTER)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_COUNTRY, RegionType::ESTONIA)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_SHOW_CHECKOUT_LOGO, true)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_MIN_ORDER_AMOUNT, 36)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_MAX_ORDER_AMOUNT, 10000)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_X_MIN_ORDER_AMOUNT, 60)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_X_MAX_ORDER_AMOUNT, 10000)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_PAY_LATER_MIN_ORDER_AMOUNT, 36)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_PAY_LATER_MAX_ORDER_AMOUNT, 10000)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_MONTHLY_ENABLED_enable_estomonthly, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_MONTHLY_URL, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_MONTHLY_DESCRIPTION, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_MONTHLY_LOGO, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_CHECKOUT_LOGO, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_X_IS_ACTIVE, false)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_X_CHECKOUT_LOGO, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_PAY_LATER_IS_ACTIVE, false)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_PAY_LATER_CHECKOUT_LOGO, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_PAY_IS_ACTIVE, false)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_PAY_CHECKOUT_LOGO, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_IS_ACTIVE, false)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_CHECKOUT_LOGO, '')
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT_EE, false)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT_LV, false)
            || !Configuration::updateValue(ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT_LT, false)
            || !$this->createBanksTable();
    }

    private function createOrderStates()
    {
        $paymentTypes = ['ESTO', 'ESTO_X', 'ESTO_PAY_LATER', 'ESTO_PAY', 'ESTO_PAY_CARD_PAYMENT'];

        foreach ($paymentTypes as $estoPayment) {
            $idOrderState = (int) Configuration::get($estoPayment . '_OS_PROCESSING', 0);
            $tmpOrderState = new OrderState($idOrderState);

            if (Validate::isLoadedObject($tmpOrderState)) {
                continue;
            }

            $orderState = new OrderState();
            $orderState->name = array();

            foreach (Language::getLanguages() as $language) {
                if (Tools::strtolower($language['iso_code']) === 'et') {
                    $text = $estoPayment === 'ESTO_PAY' ? ' makse' : ' järelmaks';
                    $orderState->name[$language['id_lang']] = str_replace('_', ' ', $estoPayment) . $text . ' pooleli';
                } else {
                    $text = $estoPayment === 'ESTO_PAY' ? ' payment' : ' hire-purchase';
                    $orderState->name[$language['id_lang']] = str_replace('_', ' ', $estoPayment) . $text . ' processing';
                }
            }

            $orderState->send_email = false;
            $orderState->color = '#4169E1';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = false;
            $orderState->unremovable = true;
            $orderState->invoice = false;

            $orderState->add();
            Configuration::updateValue($estoPayment . '_OS_PROCESSING', (int)$orderState->id);

        }
    }

    // To prevent double orders in same cart
    private function addUniqueConstraintToCartIdInOrdersTable()
    {
        $sql = 'ALTER TABLE ' . _DB_PREFIX_ . 'orders ADD CONSTRAINT UC_Cart UNIQUE (id_cart)';
        try {
            Db::getInstance()->execute($sql);
        } catch (Exception $e) {
            PrestaShopLogger::addLog('Could not create unique index ( ESTO module )');
        }

    }

    public function createBanksTable(){
        $db = Db::getInstance();
        $query = 'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.'esto_banks
		(
            `id` int(11) NOT NULL auto_increment,
            `data` text,
            `date_update` timestamp DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            PRIMARY KEY  (`id`)
		) ENGINE='.(defined('_MYSQL_ENGINE_') ? _MYSQL_ENGINE_ : "MyISAM").'  DEFAULT CHARSET=utf8;';

        $db->Execute($query);
        return true;
    }

    private function installDefaultLanguageConfiguration()
    {
        $defaults = [
            'et' => [
                ConfigKeysHelper::ESTO_PAY_CHECKOUT_LOGO => 'ee/logo-banklink.png',
                ConfigKeysHelper::ESTO_PAY_LATER_CHECKOUT_LOGO => 'ee/logo-pay-later.png',
            ],
        ];

        $languages = Language::getLanguages();

        $result = true;

        foreach($languages as $language) {
            $iso_code = $language['iso_code'];
            $id_lang = $language['id_lang'];

            if(!isset($defaults[$iso_code])) {
                continue;
            }

            $values = $defaults[$iso_code];

            foreach($values as $key => $value) {
                $result &= Configuration::updateValue($key, [$id_lang => $value]);
            }
        }

        return $result;
    }

    private function uninstallBanksTable(){
        Db::getInstance()->Execute('DROP TABLE IF EXISTS '._DB_PREFIX_.'esto_banks');
    }

    private function removeUniqueConstraintToCartIdInOrdersTable()
    {
        $sql = 'ALTER TABLE ' . _DB_PREFIX_ . 'orders DROP CONSTRAINT UC_Cart;';
        try {
            Db::getInstance()->execute($sql);
        } catch (Exception $e) {
            PrestaShopLogger::addLog('Could not delete unique index ( ESTO module )');
        }
    }
}
