<?php
/**
 * 2007-2017 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2017 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * EstoPaymentModuleFrontController Controller
 * Responsible for handling payments/oreders
 */
class EstoPaymentModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        $module = $this->module;

        // Return to order page when payment option is not selected / is illegal
        if (!Tools::getIsset('payment-option') ||
            !in_array(Tools::getValue('payment-option'), [
                PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::REGULAR],
                PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X],
                PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::PAY_LATER],
                PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK],
                PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD]
            ]) ||
            (PaymentScheduleTypeType::ESTO_X === Tools::getValue('payment-option') && $this->context->cart->getOrderTotal() < $module::ESTO_X_MIN_AMOUNT)
        ) {
            Tools::redirect($this->context->link->getPageLink('order'));
        }

        $estoPayment = Tools::getValue('payment-option');
        $cart = $this->context->cart;

        $totalAmount = $cart->getOrderTotal();

        $customer = new Customer($cart->id_customer);
        $billingAddress = new Address($cart->id_address_invoice);
        $connectionMode = Configuration::get(ConfigKeysHelper::ESTO_CONNECTION_MODE);
        $message = $connectionMode === ConnectionModeType::TEST ? 'TEST MODE' : null;
        $createOrder = Configuration::get(ConfigKeysHelper::ESTO_CREATE_ORDER);
        $reference = $cart->id;

        if ($createOrder === CreateOrderType::BEFORE) {
            $this->module->validateOrder(
                $cart->id,
                Configuration::get($estoPayment . '_OS_PROCESSING'),
                0,
                str_replace('_', ' ', $estoPayment),
                $message,
                [],
                null,
                false,
                $customer->secure_key
            );
            $reference = $this->module->currentOrder;
        }

        $api_details = $this->getApiData($billingAddress, $module);

        $data = [
            'amount' => round($totalAmount, 2),
            'reference' => (string)$reference,
            'return_url' => $this->context->link->getModuleLink($this->module->name, 'validation',
                ['esto_payment' => ($estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD]) ? PaymentScheduleTypeType::ESTO_PAY : $estoPayment, 'api_country' => $api_details['country_iso']], true),
            'notification_url' => $this->context->link->getModuleLink($this->module->name, 'validation',
                ['esto_payment' => ($estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD]) ? PaymentScheduleTypeType::ESTO_PAY : $estoPayment, 'api_country' => $api_details['country_iso']], true),
            'cancel_url' => $this->context->link->getModuleLink($this->module->name, 'validation',
                ['esto_payment' => ($estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD]) ? PaymentScheduleTypeType::ESTO_PAY : $estoPayment, 'api_country' => $api_details['country_iso']], true),
            'connection_mode' => $connectionMode,
        ];

        if ($billingAddress) {
            $data['customer'] = [
                'first_name' => $billingAddress->firstname,
                'last_name' => $billingAddress->lastname,
                'email' => $customer->email,
                'phone' => !empty($billingAddress->phone_mobile) ? $billingAddress->phone_mobile : $billingAddress->phone,
                'address' => $billingAddress->address1,
                'city' => $billingAddress->city,
                'post_code' => $billingAddress->postcode,
            ];
        }

        foreach ($data['customer'] as $key => $val) {
            if (is_null($val) || $val === '') {
                unset($data['customer'][$key]);
            }
        }

        $products = $cart->getProducts(true);
        if ($products && count($products)) {
            $items = [];

            foreach ($products as $product) {
                $items[] = [
                    'name' => $product['name'],
                    'unit_price' => round($product['price_wt'], 2),
                    'quantity' => $product['cart_quantity'],
                ];
            }
            $data['items'] = $items;
        }

        switch ($estoPayment) {
            case PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X]:
                $data['schedule_type'] = PaymentScheduleTypeType::ESTO_X;
                break;
            case PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::PAY_LATER]:
                $data['schedule_type'] = PaymentScheduleTypeType::PAY_LATER;
                break;
            case PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK]:
                $data['schedule_type'] = PaymentScheduleTypeType::ESTO_PAY;
                break;
            case PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD]:
                $data['schedule_type'] = PaymentScheduleTypeType::ESTO_PAY;
                $data['payment_method_key'] = 'STRIPE_CARD';
                break;
        }

        if(Tools::getIsset('esto-bank')) {
            $data['payment_method_key'] = Tools::getValue('esto-bank');
        }
        if(Tools::getIsset('esto-type')) {
            $data['payment_provider'] = Tools::getValue('esto-type');
        }

        $api = new EstoAPI($api_details['url'], $api_details['user'], $api_details['pass'], $api_details['country_iso']);

        Tools::redirect($api->getPurchaseRedirect($data));
    }

    protected function getApiData($billingAddress, $module)
    {
        $api_details = null;

        $api_details = $module::getApiDataByAddress($billingAddress);
        if (!empty($api_details)) {
            return $api_details;
        }

        $estoPayment = Tools::getValue('payment-option');
        if (PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK] === $estoPayment) {
            $bankCountryCode = (Tools::getValue('esto-bank-country-code'));
            if (!empty($bankCountryCode)) {
                $api_details = $module::getApiDataByCountryCode($bankCountryCode);
            }
        }
        if (!empty($api_details)) {
            return $api_details;
        }

        $api_details = $module::getDefaultCountryApiData();

        return $api_details;
    }
}
